
// Use Nes_Apu to play random tones. Write output to sound file "out.wav".

#include "Nes_Apu.h"
#include "Blip_Buffer.h"
#include "Wave_Writer.hpp"

#include <stdlib.h>

int main()
{
	long samples_per_sec = 44100;  // 44.1 kHz sample rate
	long clocks_per_sec = 1789773; // 1.79 MHz clock rate
	
	Nes_Apu apu;

	// Set up buffer
	Blip_Buffer buf;
	if ( buf.sample_rate( samples_per_sec ) )
		return EXIT_FAILURE; // out of memory
	buf.clock_rate( clocks_per_sec );
	
	// Route APU output to buffer
	apu.output( &buf );
	
	// Synthesize sound and write to sound file
	Wave_Writer wave( samples_per_sec );
	blip_time_t time = 0;
	int volume = 1;
	for ( int n = 30; n--; )
	{
		// Generate sound in 1/10 second chunks
		blip_time_t end_time = clocks_per_sec / 10;
		
		while ( time < end_time )
		{
			// Make random tones
			volume--;
			apu.write_register( time, 0x4015, 0x01 );
			apu.write_register( time, 0x4000, 0xb0 + volume );
			if ( volume == 0 )
			{
				volume = 16;
				apu.write_register( time, 0x4001, 0x7f );
				int freq = (rand() & 0x3ff) | 0x1100;
				apu.write_register( time, 0x4002, freq & 0xff );
				apu.write_register( time, 0x4003, freq >> 8 );
			}
			
			time += clocks_per_sec / 60; // 1/60 second between volume changes
		}
		
		// End time frames. The APU's frame *must* be ended first.
		time -= end_time;
		apu.end_frame( end_time );
		buf.end_frame( end_time );
		
		// Samples from the frame can now be read out of the buffer, or they
		// can be allowed to accumulate and read out later. Here they are
		// allowed to accumulate until 1/2 second of samples are available
		// before any are read out.
		if ( buf.samples_avail() >= samples_per_sec / 2 )
		{
			size_t const out_size = 1024;
			blip_sample_t out_buf [out_size];
			
			// Keep reading samples into buffer and writing to sound file until
			// no more are available.
			long samples_read;
			while ( (samples_read = buf.read_samples( out_buf, out_size )) != 0 )
				wave.write( out_buf, samples_read );
		}
	}
	
	return 0;
}

