#include "../emu/h/types.h"
#include "../emu/h/emuFileHandling.h"
#include "h/cpu.h"
#include "h/nesGeneral.h"
#include "h/registers.h"

namespace Reminesce
{

extern ui cpuCycles, cpuTargetCycles;
extern s32 g_badOp;

extern struct tCPU cpu;

//for regs
extern u16 pad[2];
extern u8 ButtonShift1, ButtonShift2, Strobing;

namespace cpuLogCode
{

#define loggedCpu

#include "h/Addressing.h"
#include "h/instructions.h"

using namespace Reminesce;

inline void runOp(u8 opcode)
{
	switch (opcode)
	{
	case 0x69: adrImm()	; AddcpuCycles (2); ADC(); break;
	case 0x65: adrZp()	; AddcpuCycles (3); ADC(); break;
	case 0x75: adrZpx()	; AddcpuCycles (4); ADC(); break;
	case 0x6D: adrAb()	; AddcpuCycles (4); ADC(); break;
	case 0x7D: adrAbxp(); AddcpuCyclesp(4); ADC(); break;
	case 0x79: adrAbyp(); AddcpuCyclesp(4); ADC(); break;
	case 0x61: adrInx()	; AddcpuCycles (6); ADC(); break;
	case 0x71: adrInyp(); AddcpuCyclesp(5); ADC(); break;
	 
	case 0x29: adrImm()	; AddcpuCycles (2); AND(); break;
	case 0x25: adrZp()	; AddcpuCycles (2); AND(); break;
	case 0x35: adrZpx()	; AddcpuCycles (3); AND(); break;
	case 0x2D: adrAb()	; AddcpuCycles (4); AND(); break;
	case 0x3D: adrAbxp(); AddcpuCyclesp(4); AND(); break;
	case 0x39: adrAbyp(); AddcpuCyclesp(4); AND(); break;
	case 0x21: adrInx()	; AddcpuCycles (6); AND(); break;
	case 0x31: adrInyp(); AddcpuCyclesp(5); AND(); break;
	
	case 0x0A: adrImp()	; AddcpuCycles (2); ASLA();break;
	case 0x06: adrZp()	; AddcpuCycles (5); ASL(); break;
	case 0x16: adrZpx()	; AddcpuCycles (6); ASL(); break;
	case 0x0E: adrAb()	; AddcpuCycles (6); ASL(); break;
	case 0x1E: adrAbx()	; AddcpuCycles (7); ASL(); break;
	
	case 0x24: adrZp()	; AddcpuCycles (3); BIT(); break;
	case 0x2C: adrAb()	; AddcpuCycles (4); BIT(); break;
	
	case 0x10:							  ; BPL(); break;
	case 0x30:							  ; BMI(); break;
	case 0x50:							  ; BVC(); break;
	case 0x70:							  ; BVS(); break;
	case 0x90:							  ; BCC(); break;
	case 0xB0:							  ; BCS(); break;
	case 0xD0:							  ; BNE(); break;
	case 0xF0:							  ; BEQ(); break;
	
	case 0x00: adrImp()	; AddcpuCycles (7); BRK(); break;
	
	case 0xC9: adrImm()	; AddcpuCycles (2); CMP(); break;
	case 0xC5: adrZp()	; AddcpuCycles (3); CMP(); break;
	case 0xD5: adrZpx()	; AddcpuCycles (4); CMP(); break;
	case 0xCD: adrAb()	; AddcpuCycles (4); CMP(); break;
	case 0xDD: adrAbxp(); AddcpuCyclesp(4); CMP(); break;
	case 0xD9: adrAbyp(); AddcpuCyclesp(4); CMP(); break;
	case 0xC1: adrInx()	; AddcpuCycles (6); CMP(); break;
	case 0xD1: adrInyp(); AddcpuCyclesp(5); CMP(); break;
	
	case 0xE0: adrImm()	; AddcpuCycles (2); CPX(); break;
	case 0xE4: adrZp()	; AddcpuCycles (3); CPX(); break;
	case 0xEC: adrAb()	; AddcpuCycles (4); CPX(); break;
	
	case 0xC0: adrImm()	; AddcpuCycles (2); CPY(); break;
	case 0xC4: adrZp()	; AddcpuCycles (3); CPY(); break;
	case 0xCC: adrAb()	; AddcpuCycles (4); CPY(); break;
	
	case 0xC6: adrZp()	; AddcpuCycles (5); DEC(); break;
	case 0xD6: adrZpx()	; AddcpuCycles (6); DEC(); break;
	case 0xCE: adrAb()	; AddcpuCycles (6); DEC(); break;
	case 0xDE: adrAbx()	; AddcpuCycles (7); DEC(); break;
	
	case 0x49: adrImm()	; AddcpuCycles (2); EOR(); break;
	case 0x45: adrZp()	; AddcpuCycles (3); EOR(); break;
	case 0x55: adrZpx()	; AddcpuCycles (4); EOR(); break;
	case 0x4D: adrAb()	; AddcpuCycles (4); EOR(); break;
	case 0x5D: adrAbxp(); AddcpuCyclesp(4); EOR(); break;
	case 0x59: adrAbyp(); AddcpuCyclesp(4); EOR(); break;
	case 0x41: adrInx()	; AddcpuCycles (6); EOR(); break;
	case 0x51: adrInyp(); AddcpuCyclesp(5); EOR(); break;
	
	case 0x18: adrImp()	; AddcpuCycles (2); CLC(); break;
	case 0x38: adrImp()	; AddcpuCycles (2); SEC(); break;
	case 0x58: adrImp()	; AddcpuCycles (2); CLI(); break;
	case 0x78: adrImp()	; AddcpuCycles (2); SEI(); break;
	case 0xB8: adrImp()	; AddcpuCycles (2); CLV(); break;
	case 0xD8: adrImp()	; AddcpuCycles (2); CLD(); break;
	case 0xF8: adrImp()	; AddcpuCycles (2); SED(); break;
	
	case 0xE6: adrZp()	; AddcpuCycles (5); INC(); break;
	case 0xF6: adrZpx()	; AddcpuCycles (6); INC(); break;
	case 0xEE: adrAb()	; AddcpuCycles (6); INC(); break;
	case 0xFE: adrAbx()	; AddcpuCycles (7); INC(); break;
	
	case 0x4C:			; AddcpuCycles (3); JMP(); break;
	case 0x6C:			; AddcpuCycles (5); JMPIND(); break;
	
	case 0x20:			; AddcpuCycles (6); JSR(); break;
	
	case 0xA9: adrImm()	; AddcpuCycles (2); LDA(); break;
	case 0xA5: adrZp()	; AddcpuCycles (3); LDA(); break;
	case 0xB5: adrZpx()	; AddcpuCycles (4); LDA(); break;
	case 0xAD: adrAb()	; AddcpuCycles (4); LDA(); break;
	case 0xBD: adrAbxp(); AddcpuCyclesp(4); LDA(); break;
	case 0xB9: adrAbyp(); AddcpuCyclesp(4); LDA(); break;
	case 0xA1: adrInx()	; AddcpuCycles (6); LDA(); break;
	case 0xB1: adrInyp(); AddcpuCyclesp(5); LDA(); break;
	
	case 0xA2: adrImm()	; AddcpuCycles (2); LDX(); break;
	case 0xA6: adrZp()	; AddcpuCycles (3); LDX(); break;
	case 0xB6: adrZpy()	; AddcpuCycles (4); LDX(); break;
	case 0xAE: adrAb()	; AddcpuCycles (4); LDX(); break;
	case 0xBE: adrAbyp(); AddcpuCyclesp(4); LDX(); break;
	
	case 0xA0: adrImm()	; AddcpuCycles (2); LDY(); break;
	case 0xA4: adrZp()	; AddcpuCycles (3); LDY(); break;
	case 0xB4: adrZpx()	; AddcpuCycles (4); LDY(); break;
	case 0xAC: adrAb()	; AddcpuCycles (4); LDY(); break;
	case 0xBC: adrAbxp(); AddcpuCyclesp(4); LDY(); break;
	
	case 0x4A: adrImp()	; AddcpuCycles (2); LSRA();break;
	case 0x46: adrZp()	; AddcpuCycles (5); LSR(); break;
	case 0x56: adrZpx()	; AddcpuCycles (6); LSR(); break;
	case 0x4E: adrAb()	; AddcpuCycles (6); LSR(); break;
	case 0x5E: adrAbx()	; AddcpuCycles (7); LSR(); break;
	
	case 0xEA: adrImp()	; AddcpuCycles (2); NOP(); break;
	
	case 0x09: adrImm()	; AddcpuCycles (2); ORA(); break;
	case 0x05: adrZp()	; AddcpuCycles (2); ORA(); break;
	case 0x15: adrZpx()	; AddcpuCycles (3); ORA(); break;
	case 0x0D: adrAb()	; AddcpuCycles (4); ORA(); break;
	case 0x1D: adrAbxp(); AddcpuCyclesp(4); ORA(); break;
	case 0x19: adrAbyp(); AddcpuCyclesp(4); ORA(); break;
	case 0x01: adrInx()	; AddcpuCycles (6); ORA(); break;
	case 0x11: adrInyp(); AddcpuCyclesp(5); ORA(); break;
	
	case 0xAA: adrImp()	; AddcpuCycles (2); TAX(); break;
	case 0x8A: adrImp()	; AddcpuCycles (2); TXA(); break;
	case 0xCA: adrImp()	; AddcpuCycles (2); DEX(); break;
	case 0xE8: adrImp()	; AddcpuCycles (2); INX(); break;
	case 0xA8: adrImp()	; AddcpuCycles (2); TAY(); break;
	case 0x98: adrImp()	; AddcpuCycles (2); TYA(); break;
	case 0x88: adrImp()	; AddcpuCycles (2); DEY(); break;
	case 0xC8: adrImp()	; AddcpuCycles (2); INY(); break;
	
	case 0x2A: adrImp()	; AddcpuCycles (2); ROLA();break;
	case 0x26: adrZp()	; AddcpuCycles (5); ROL(); break;
	case 0x36: adrZpx()	; AddcpuCycles (6); ROL(); break;
	case 0x2E: adrAb()	; AddcpuCycles (6); ROL(); break;
	case 0x3E: adrAbx()	; AddcpuCycles (7); ROL(); break;
	
	case 0x6A: adrImp()	; AddcpuCycles (2); RORA();break;
	case 0x66: adrZp()	; AddcpuCycles (5); ROR(); break;
	case 0x76: adrZpx()	; AddcpuCycles (6); ROR(); break;
	case 0x6E: adrAb()	; AddcpuCycles (6); ROR(); break;
	case 0x7E: adrAbx()	; AddcpuCycles (7); ROR(); break;
	
	case 0x40:			; AddcpuCycles (6); RTI(); break;
	case 0x60:			; AddcpuCycles (6); RTS(); break;
	
	case 0xE9: adrImm()	; AddcpuCycles (2); SBC(); break;
	case 0xE5: adrZp()	; AddcpuCycles (3); SBC(); break;
	case 0xF5: adrZpx()	; AddcpuCycles (4); SBC(); break;
	case 0xED: adrAb()	; AddcpuCycles (4); SBC(); break;
	case 0xFD: adrAbxp(); AddcpuCyclesp(4); SBC(); break;
	case 0xF9: adrAbyp(); AddcpuCyclesp(4); SBC(); break;
	case 0xE1: adrInx()	; AddcpuCycles (6); SBC(); break;
	case 0xF1: adrInyp(); AddcpuCyclesp(5); SBC(); break;
	
	case 0x85: adrZp()	; AddcpuCycles (3); STA(); break;
	case 0x95: adrZpx()	; AddcpuCycles (4); STA(); break;
	case 0x8D: adrAb()	; AddcpuCycles (4); STA(); break;
	case 0x9D: adrAbx()	; AddcpuCycles (5); STA(); break;
	case 0x99: adrAby()	; AddcpuCycles (5); STA(); break;
	case 0x81: adrInx()	; AddcpuCycles (6); STA(); break;
	case 0x91: adrIny()	; AddcpuCycles (6); STA(); break;
	
	case 0x9A: adrImp()	; AddcpuCycles (2); TXS(); break;
	case 0xBA: adrImp()	; AddcpuCycles (2); TSX(); break;
	case 0x48: adrImp()	; AddcpuCycles (3); PHA(); break;
	case 0x68: adrImp()	; AddcpuCycles (4); PLA(); break;
	case 0x08: adrImp()	; AddcpuCycles (3); PHP(); break;
	case 0x28: adrImp()	; AddcpuCycles (4); PLP(); break;
	
	case 0x86: adrZp()	; AddcpuCycles (3); STX(); break;
	case 0x96: adrZpy()	; AddcpuCycles (4); STX(); break;
	case 0x8E: adrAb()	; AddcpuCycles (4); STX(); break;
	
	case 0x84: adrZp()	; AddcpuCycles (3); STY(); break;
	case 0x94: adrZpx()	; AddcpuCycles (4); STY(); break;
	case 0x8C: adrAb()	; AddcpuCycles (4); STY(); break;

	default: g_badOp = cpu.pc; adrImp(); AddcpuCycles (2); break;
	}
}

void runCpuLog(int cycles)
{
	cpuTargetCycles += cycles;
	while (cpuCycles < cpuTargetCycles)
	{
 		runOp(pRead(cpu.pc));
	}
}
}

}
